@@ -1,19 +1,25 @@
+//For quick-error with large enums
+#![recursion_limit = "128"]
+
 //! A user friendly crate that allows you to share memory between __processes__
 //!
 //! For help on how to get started, take a look at the examples !
 
-#[macro_use]
-extern crate cfg_if;
-#[macro_use]
-extern crate enum_primitive;
+use ::cfg_if::*;
+// Re-export proc macros
+#[doc(hidden)]
+pub use shared_memory_derive::*;
 
 use std::ffi::OsStr;
-use std::path::{PathBuf, Path};
-use std::fs::{File};
-use std::fs::remove_file;
-use std::slice;
-use std::os::raw::c_void;
 use std::fmt;
+use std::fs::remove_file;
+use std::fs::File;
+use std::os::raw::c_void;
+use std::path::{Path, PathBuf};
+use std::slice;
+
+mod error;
+pub use error::*;
 
 //Lock definitions
 mod locks;
@@ -30,7 +36,7 @@ cfg_if! {
         use windows as os_impl;
     } else if #[cfg(any(target_os="freebsd", target_os="linux", target_os="macos"))] {
         mod nix;
-        use nix as os_impl;
+        use crate::nix as os_impl;
     } else {
         compile_error!("shared_memory isnt implemented for this platform...");
     }
@@ -40,9 +46,6 @@ cfg_if! {
 //locks/events in the metadata section.
 const ADDR_ALIGN: u8 = 4;
 
-//TODO : Replace this with proper error handling, failure crate maybe
-type Result<T> = std::result::Result<T, Box<std::error::Error>>;
-
 ///Defines different variants to specify timeouts
 pub enum Timeout {
     ///Wait forever for an event to be signaled
@@ -81,28 +84,35 @@ pub struct SharedMem {
     user_ptr: *mut c_void,
 }
 impl SharedMem {
-
     ///Creates a memory mapping with no link file of specified size controlled by a single lock.
-    pub fn create(lock_type: LockType, size: usize) -> Result<SharedMem> {
-        SharedMemConf::new()
+    pub fn create(lock_type: LockType, size: usize) -> Result<SharedMem, SharedMemError> {
+        SharedMemConf::default()
             .set_size(size)
-            .add_lock(lock_type, 0, size).unwrap().create()
+            .add_lock(lock_type, 0, size)
+            .unwrap()
+            .create()
     }
 
-    pub fn open(unique_id: &str) -> Result<SharedMem> {
-        SharedMemConf::new()
-            .set_os_path(unique_id)
-            .open()
+    pub fn open(unique_id: &str) -> Result<SharedMem, SharedMemError> {
+        SharedMemConf::default().set_os_path(unique_id).open()
     }
 
-    pub fn create_linked<I: AsRef<OsStr>>(new_link_path: I, lock_type: LockType, size: usize) -> Result<SharedMem> {
-        SharedMemConf::new()
+    pub fn create_linked<I: AsRef<OsStr>>(
+        new_link_path: I,
+        lock_type: LockType,
+        size: usize,
+    ) -> Result<SharedMem, SharedMemError> {
+        SharedMemConf::default()
             .set_link_path(new_link_path.as_ref())
             .set_size(size)
-            .add_lock(lock_type, 0, size).unwrap().create()
+            .add_lock(lock_type, 0, size)
+            .unwrap()
+            .create()
     }
-    pub fn open_linked<I: AsRef<OsStr>>(existing_link_path: I) -> Result<SharedMem> {
-        SharedMemConf::new()
+    pub fn open_linked<I: AsRef<OsStr>>(
+        existing_link_path: I,
+    ) -> Result<SharedMem, SharedMemError> {
+        SharedMemConf::default()
             .set_link_path(existing_link_path.as_ref())
             .open()
     }
@@ -143,29 +153,35 @@ impl SharedMem {
     pub fn get_ptr(&self) -> *mut c_void {
         self.user_ptr
     }
+
+    #[inline]
+    pub fn is_owner(&self) -> bool {
+        self.conf.is_owner()
+    }
 }
 impl Drop for SharedMem {
-
     ///Deletes the SharedMemConf artifacts
     fn drop(&mut self) {
-
         //Close the openned link file
-        drop(&self.link_file);
+        drop(self.link_file.take());
 
         //Delete link file if we own it
         if self.conf.is_owner() {
             if let Some(ref file_path) = self.conf.get_link_path() {
                 if file_path.is_file() {
-                    match remove_file(file_path) {_=>{},};
+                    match remove_file(file_path) {
+                        _ => {}
+                    };
                 }
             }
         }
     }
 }
 impl fmt::Display for SharedMem {
-
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "
+        write!(
+            f,
+            "
         Created : {}
         link : \"{}\"
         os_id : \"{}\"
@@ -176,7 +192,9 @@ impl fmt::Display for SharedMem {
         MetaAddr : {:p}
         UserAddr : {:p}",
             self.conf.is_owner(),
-            self.get_link_path().unwrap_or(&PathBuf::from("[NONE]")).to_string_lossy(),
+            self.get_link_path()
+                .unwrap_or(&PathBuf::from("[NONE]"))
+                .to_string_lossy(),
             self.get_os_path(),
             self.get_metadata_size(),
             self.get_size(),
@@ -188,16 +206,16 @@ impl fmt::Display for SharedMem {
     }
 }
 impl ReadLockable for SharedMem {
-    fn rlock<D: SharedMemCast>(&self, lock_index: usize) -> Result<ReadLockGuard<D>> {
-
+    fn rlock<D: SharedMemCast>(
+        &self,
+        lock_index: usize,
+    ) -> Result<ReadLockGuard<D>, SharedMemError> {
         let lock: &GenericLock = self.conf.get_lock(lock_index);
 
         //Make sure that we can cast our memory to the type
         let type_size = std::mem::size_of::<D>();
         if type_size > lock.length {
-            return Err(From::from(
-                format!("Tried to map type of {} bytes to a lock holding only {} bytes", type_size, lock.length)
-            ));
+            return Err(SharedMemError::RangeDoesNotFit(type_size, lock.length));
         }
 
         //Return data wrapped in a lock
@@ -209,20 +227,20 @@ impl ReadLockable for SharedMem {
                     lock.interface,
                     &mut (*lock.lock_ptr),
                 )
-            }
+            },
         )
     }
 
-    fn rlock_as_slice<D: SharedMemCast>(&self, lock_index: usize) -> Result<ReadLockGuardSlice<D>>{
-
+    fn rlock_as_slice<D: SharedMemCast>(
+        &self,
+        lock_index: usize,
+    ) -> Result<ReadLockGuardSlice<D>, SharedMemError> {
         let lock: &GenericLock = self.conf.get_lock(lock_index);
 
         //Make sure that we can cast our memory to the slice
         let item_size = std::mem::size_of::<D>();
         if item_size > lock.length {
-            return Err(From::from(
-                format!("Tried to map type of {} bytes to a lock holding only {} bytes", item_size, lock.length)
-            ));
+            return Err(SharedMemError::RangeDoesNotFit(item_size, lock.length));
         }
         let num_items: usize = lock.length / item_size;
 
@@ -235,21 +253,21 @@ impl ReadLockable for SharedMem {
                     lock.interface,
                     &mut (*lock.lock_ptr),
                 )
-            }
+            },
         )
     }
 }
 impl WriteLockable for SharedMem {
-    fn wlock<D: SharedMemCast>(&mut self, lock_index: usize) -> Result<WriteLockGuard<D>> {
-
+    fn wlock<D: SharedMemCast>(
+        &mut self,
+        lock_index: usize,
+    ) -> Result<WriteLockGuard<D>, SharedMemError> {
         let lock: &GenericLock = self.conf.get_lock(lock_index);
 
         //Make sure that we can cast our memory to the type
         let type_size = std::mem::size_of::<D>();
         if type_size > lock.length {
-            return Err(From::from(
-                format!("Tried to map type of {} bytes to a lock holding only {} bytes", type_size, lock.length)
-            ));
+            return Err(SharedMemError::RangeDoesNotFit(type_size, lock.length));
         }
 
         //Return data wrapped in a lock
@@ -261,20 +279,20 @@ impl WriteLockable for SharedMem {
                     lock.interface,
                     &mut (*lock.lock_ptr),
                 )
-            }
+            },
         )
     }
 
-    fn wlock_as_slice<D: SharedMemCast>(&mut self, lock_index: usize) -> Result<WriteLockGuardSlice<D>> {
-
+    fn wlock_as_slice<D: SharedMemCast>(
+        &mut self,
+        lock_index: usize,
+    ) -> Result<WriteLockGuardSlice<D>, SharedMemError> {
         let lock: &GenericLock = self.conf.get_lock(lock_index);
 
         //Make sure that we can cast our memory to the slice
         let item_size = std::mem::size_of::<D>();
         if item_size > lock.length {
-            return Err(From::from(
-                format!("Tried to map type of {} bytes to a lock holding only {} bytes", item_size, lock.length)
-            ));
+            return Err(SharedMemError::RangeDoesNotFit(item_size, lock.length));
         }
         //Calculate how many items our slice will have
         let num_items: usize = lock.length / item_size;
@@ -284,57 +302,65 @@ impl WriteLockable for SharedMem {
             //Unsafe required to cast shared memory to array
             unsafe {
                 WriteLockGuardSlice::lock(
-                    slice::from_raw_parts_mut((lock.data_ptr as usize + 0) as *mut D, num_items),
+                    slice::from_raw_parts_mut((lock.data_ptr as usize) as *mut D, num_items),
                     lock.interface,
                     &mut (*lock.lock_ptr),
                 )
-            }
+            },
         )
     }
 }
 impl ReadRaw for SharedMem {
     unsafe fn get_raw<D: SharedMemCast>(&self) -> &D {
         let user_data = self.os_data.map_ptr as usize + self.conf.get_metadata_size();
-        return &(*(user_data as *const D))
+        &(*(user_data as *const D))
     }
 
     unsafe fn get_raw_slice<D: SharedMemCast>(&self) -> &[D] {
         //Make sure that we can cast our memory to the slice
         let item_size = std::mem::size_of::<D>();
         if item_size > self.conf.get_size() {
-            panic!("Tried to map type of {} bytes to a lock holding only {} bytes", item_size, self.conf.get_size());
+            panic!(
+                "Tried to map type of {} bytes to a lock holding only {} bytes",
+                item_size,
+                self.conf.get_size()
+            );
         }
         let num_items: usize = self.conf.get_size() / item_size;
         let user_data = self.os_data.map_ptr as usize + self.conf.get_metadata_size();
 
-        return slice::from_raw_parts(user_data as *const D, num_items);
+        slice::from_raw_parts(user_data as *const D, num_items)
     }
 }
 impl WriteRaw for SharedMem {
     unsafe fn get_raw_mut<D: SharedMemCast>(&mut self) -> &mut D {
         let user_data = self.os_data.map_ptr as usize + self.conf.get_metadata_size();
-        return &mut (*(user_data as *mut D))
+        &mut (*(user_data as *mut D))
     }
-    unsafe fn get_raw_slice_mut<D: SharedMemCast>(&mut self) -> &mut[D] {
+    unsafe fn get_raw_slice_mut<D: SharedMemCast>(&mut self) -> &mut [D] {
         //Make sure that we can cast our memory to the slice
         let item_size = std::mem::size_of::<D>();
         if item_size > self.conf.get_size() {
-            panic!("Tried to map type of {} bytes to a lock holding only {} bytes", item_size, self.conf.get_size());
+            panic!(
+                "Tried to map type of {} bytes to a lock holding only {} bytes",
+                item_size,
+                self.conf.get_size()
+            );
         }
         let num_items: usize = self.conf.get_size() / item_size;
         let user_data = self.os_data.map_ptr as usize + self.conf.get_metadata_size();
 
-        return slice::from_raw_parts_mut(user_data as *mut D, num_items);
+        slice::from_raw_parts_mut(user_data as *mut D, num_items)
     }
 }
 impl EventSet for SharedMem {
-    fn set(&mut self, event_index: usize, state: EventState) -> Result<()> {
+    fn set(&mut self, event_index: usize, state: EventState) -> Result<(), SharedMemError> {
         let lock: &GenericEvent = self.conf.get_event(event_index);
         lock.interface.set(lock.ptr, state)
     }
 }
 impl EventWait for SharedMem {
-    fn wait(&self, event_index: usize, timeout: Timeout) -> Result<()> {
+    fn wait(&self, event_index: usize, timeout: Timeout) -> Result<(), SharedMemError> {
         let lock: &GenericEvent = self.conf.get_event(event_index);
         lock.interface.wait(lock.ptr, timeout)
     }
