@@ -1,4 +1,4 @@
-use super::libc::{
+use ::libc::{
     pthread_mutex_t,
     timespec,
     clock_gettime,
@@ -10,9 +10,9 @@ use super::libc::{
     c_int,
 };
 
-use super::std::ptr::{null_mut};
+use std::ptr::{null_mut};
 
-pub fn pthread_mutex_timedlock(lock: *mut pthread_mutex_t, abstime: &timespec) -> c_int {
+pub unsafe fn pthread_mutex_timedlock(lock: *mut pthread_mutex_t, abstime: &timespec) -> c_int {
 
     let mut timenow: timespec = timespec {
         tv_sec: 0,
@@ -27,17 +27,17 @@ pub fn pthread_mutex_timedlock(lock: *mut pthread_mutex_t, abstime: &timespec) -
     let mut res: c_int;
 
     loop {
-        res = unsafe {pthread_mutex_trylock(lock)};
+        res = pthread_mutex_trylock(lock);
 
         if res == EBUSY {
             // Check timeout before sleeping
-            unsafe {clock_gettime(CLOCK_REALTIME, &mut timenow)};
+            clock_gettime(CLOCK_REALTIME, &mut timenow);
             if timenow.tv_sec >= abstime.tv_sec && timenow.tv_nsec >= abstime.tv_nsec {
                 return ETIMEDOUT;
             }
 
             //Sleep for a bit
-            unsafe {nanosleep(&timesleep, null_mut())};
+            nanosleep(&timesleep, null_mut());
 
             continue;
         }
