@@ -1,20 +1,19 @@
-extern crate libc;
-extern crate nix;
+use ::cfg_if::*;
 
 cfg_if!{
     if #[cfg(target_os="linux")] {
         pub mod linux;
-        pub use self::linux::*;
-        use self::libc::pthread_mutex_timedlock;
+        pub use crate::nix::linux::*;
+        use ::libc::pthread_mutex_timedlock;
     } else if #[cfg(target_os="macos")] {
         pub mod mac;
-        pub use self::mac::*;
+        pub use crate::nix::mac::*;
     } else {
-        use self::libc::pthread_mutex_timedlock;
+        use ::libc::pthread_mutex_timedlock;
     }
 }
 
-use self::libc::{
+use ::libc::{
     timespec,
     time_t,
     c_long,
@@ -56,13 +55,14 @@ use self::libc::{
     PTHREAD_PROCESS_SHARED,
 };
 
-use self::nix::sys::mman::{mmap, munmap, shm_open, shm_unlink, ProtFlags, MapFlags};
-use self::nix::errno::Errno;
-use self::nix::sys::stat::{fstat, Mode};
-use self::nix::fcntl::OFlag;
-use self::nix::unistd::{close, ftruncate};
+use ::nix::sys::mman::{mmap, munmap, shm_open, shm_unlink, ProtFlags, MapFlags};
+use ::nix::errno::Errno;
+use ::nix::sys::stat::{fstat, Mode};
+use ::nix::fcntl::OFlag;
+use ::nix::unistd::{close, ftruncate};
 
-use super::{std,
+use crate::{
+    SharedMemError,
     LockType,
     GenericLock,
     LockImpl,
@@ -73,14 +73,12 @@ use super::{std,
     GenericEvent,
     AutoBusy,
     ManualBusy,
-    Result,
 };
 
 use std::os::raw::c_void;
 use std::os::unix::io::RawFd;
 use std::ptr::{null_mut};
 use std::mem::size_of;
-use std::time::{Duration, Instant};
 
 /*
 #[cfg(target_os="macos")]
@@ -114,9 +112,9 @@ impl Drop for MapData {
         if !self.map_ptr.is_null() {
             match unsafe {munmap(self.map_ptr as *mut _, self.map_size)} {
                 Ok(_) => {},
-                Err(e) => {
-                    println!("os_impl::Linux : Failed to munmap() shared memory mapping...");
-                    println!("{}", e);
+                Err(_e) => {
+                    //debug!("os_impl::Linux : Failed to munmap() shared memory mapping...");
+                    //debug!("{}", e);
                 },
             };
         }
@@ -127,22 +125,22 @@ impl Drop for MapData {
             if self.owner {
                 match shm_unlink(self.unique_id.as_str()) {
                     Ok(_) => {
-                        //println!("shm_unlink()");
+                        //debug!("shm_unlink()");
                     },
-                    Err(e) => {
-                        println!("os_impl::Linux : Failed to shm_unlink() shared memory name...");
-                        println!("{}", e);
+                    Err(_e) => {
+                        //debug!("os_impl::Linux : Failed to shm_unlink() shared memory name...");
+                        //debug!("{}", e);
                     },
                 };
             }
 
             match close(self.map_fd) {
                 Ok(_) => {
-                    //println!("close()");
+                    //debug!("close()");
                 },
-                Err(e) => {
-                    println!("os_impl::Linux : Failed to close() shared memory file descriptor...");
-                    println!("{}", e);
+                Err(_e) => {
+                    //debug!("os_impl::Linux : Failed to close() shared memory file descriptor...");
+                    //debug!("{}", e);
                 },
             };
         }
@@ -150,7 +148,7 @@ impl Drop for MapData {
 }
 
 //Creates a mapping specified by the uid and size
-pub fn create_mapping(unique_id: &str, map_size: usize) -> Result<MapData> {
+pub fn create_mapping(unique_id: &str, map_size: usize) -> Result<MapData, SharedMemError> {
 
     //Create shared memory file descriptor
     let shmem_fd = match shm_open(
@@ -159,8 +157,9 @@ pub fn create_mapping(unique_id: &str, map_size: usize) -> Result<MapData> {
         Mode::S_IRUSR|Mode::S_IWUSR //Permission allow user+rw
     ) {
         Ok(v) => v,
-        Err(nix::Error::Sys(Errno::EEXIST)) => return Err(From::from("NAME_EXISTS")),
-        e => return Err(From::from(format!("shm_open() failed with :\n{:?}", e))),
+        Err(nix::Error::Sys(Errno::EEXIST)) => return Err(SharedMemError::MappingIdExists),
+        Err(nix::Error::Sys(e)) => return Err(SharedMemError::MapCreateFailed(e as u32)),
+        _ => return Err(SharedMemError::UnknownOsError(0xffff_ffff)),
     };
 
     let mut new_map: MapData = MapData {
@@ -174,7 +173,8 @@ pub fn create_mapping(unique_id: &str, map_size: usize) -> Result<MapData> {
     //Enlarge the memory descriptor file size to the requested map size
     match ftruncate(new_map.map_fd, new_map.map_size as _) {
         Ok(_) => {},
-        Err(e) => return Err(From::from(format!("ftruncate() failed with :\n{}", e))),
+        Err(nix::Error::Sys(e)) => return Err(SharedMemError::UnknownOsError(e as u32)),
+        _ => return Err(SharedMemError::UnknownOsError(0xffff_ffff)),
     };
 
     //Put the mapping in our address space
@@ -188,14 +188,15 @@ pub fn create_mapping(unique_id: &str, map_size: usize) -> Result<MapData> {
         )
     } {
         Ok(v) => v as *mut c_void,
-        Err(e) => return Err(From::from(format!("mmap() failed with :\n{}", e))),
+        Err(nix::Error::Sys(e)) => return Err(SharedMemError::MapCreateFailed(e as u32)),
+        _ => return Err(SharedMemError::UnknownOsError(0xffff_ffff)),
     };
 
     Ok(new_map)
 }
 
 //Opens an existing mapping specified by its uid
-pub fn open_mapping(unique_id: &str) -> Result<MapData> {
+pub fn open_mapping(unique_id: &str) -> Result<MapData, SharedMemError> {
     //Open shared memory
     let shmem_fd = match shm_open(
         unique_id,
@@ -203,7 +204,8 @@ pub fn open_mapping(unique_id: &str) -> Result<MapData> {
         Mode::S_IRUSR
     ) {
         Ok(v) => v,
-        Err(e) => return Err(From::from(format!("shm_open() failed with :\n{}", e))),
+        Err(nix::Error::Sys(e)) => return Err(SharedMemError::MapOpenFailed(e as u32)),
+        _ => return Err(SharedMemError::UnknownOsError(0xffff_ffff)),
     };
 
     let mut new_map: MapData = MapData {
@@ -217,9 +219,8 @@ pub fn open_mapping(unique_id: &str) -> Result<MapData> {
     //Get mmap size
     new_map.map_size = match fstat(new_map.map_fd) {
         Ok(v) => v.st_size as usize,
-        Err(e) => {
-            return Err(From::from(e));
-        }
+        Err(nix::Error::Sys(e)) => return Err(SharedMemError::MapOpenFailed(e as u32)),
+        _ => return Err(SharedMemError::UnknownOsError(0xffff_ffff)),
     };
 
     //Map memory into our address space
@@ -233,31 +234,32 @@ pub fn open_mapping(unique_id: &str) -> Result<MapData> {
         )
     } {
         Ok(v) => v as *mut c_void,
-        Err(e) => return Err(From::from(format!("mmap() failed with :\n{}", e))),
+        Err(nix::Error::Sys(e)) => return Err(SharedMemError::MapOpenFailed(e as u32)),
+        _ => return Err(SharedMemError::UnknownOsError(0xffff_ffff)),
     };
 
     Ok(new_map)
 }
 
 //This functions exports our implementation for each lock type
-pub fn lockimpl_from_type(lock_type: &LockType) -> &'static LockImpl {
+pub fn lockimpl_from_type(lock_type: LockType) -> &'static dyn LockImpl {
     match lock_type {
-        &LockType::Mutex => &Mutex{},
-        &LockType::RwLock => &RwLock{},
+        LockType::Mutex => &Mutex{},
+        LockType::RwLock => &RwLock{},
     }
 }
 
 //This functions exports our implementation for each event type
-pub fn eventimpl_from_type(event_type: &EventType) -> &'static EventImpl {
+pub fn eventimpl_from_type(event_type: EventType) -> &'static dyn EventImpl {
     match event_type {
-        &EventType::AutoBusy => &AutoBusy{},
-        &EventType::ManualBusy => &ManualBusy{},
-        &EventType::Manual => &ManualGeneric{},
-        &EventType::Auto => &AutoGeneric{},
+        EventType::AutoBusy => &AutoBusy{},
+        EventType::ManualBusy => &ManualBusy{},
+        EventType::Manual => &ManualGeneric{},
+        EventType::Auto => &AutoGeneric{},
         #[cfg(target_os="linux")]
-        &EventType::AutoEventFd => &AutoEventFd{},
+        EventType::AutoEventFd => &AutoEventFd{},
         #[cfg(target_os="linux")]
-        &EventType::ManualEventFd => &ManualEventFd{},
+        EventType::ManualEventFd => &ManualEventFd{},
     }
 }
 
@@ -290,36 +292,36 @@ fn timeout_to_abstimespec(timeout: Timeout) -> timespec {
 
 /* Lock Implementations */
 
-fn new_mutex(mutex: *mut pthread_mutex_t) -> Result<()> {
+fn new_mutex(mutex: *mut pthread_mutex_t) -> Result<(), SharedMemError> {
     let mut res: libc::c_int;
 
-    let mut lock_attr: [u8; size_of::<pthread_mutexattr_t>()] = [0; size_of::<pthread_mutexattr_t>()];
+    let mut lock_attr: pthread_mutexattr_t = unsafe {std::mem::zeroed()};
 
     //Set the PTHREAD_PROCESS_SHARED attribute on our rwlock
-    res = unsafe{pthread_mutexattr_init(lock_attr.as_mut_ptr() as *mut pthread_mutexattr_t)};
+    res = unsafe{pthread_mutexattr_init(&mut lock_attr)};
     if res != 0 {
-        return Err(From::from(format!("pthread_condattr_init() : Failed with {}", res)));
+        return Err(SharedMemError::FailedToCreateLock(res as u32));
     }
-    res = unsafe{pthread_mutexattr_setpshared(lock_attr.as_mut_ptr() as *mut pthread_mutexattr_t, PTHREAD_PROCESS_SHARED)};
+    res = unsafe{pthread_mutexattr_setpshared(&mut lock_attr, PTHREAD_PROCESS_SHARED)};
     if res != 0 {
-        return Err(From::from(format!("pthread_condattr_init() : Failed with {}", res)));
+        return Err(SharedMemError::FailedToCreateLock(res as u32));
     }
     //Init the rwlock
-    res = unsafe{pthread_mutex_init(mutex, lock_attr.as_mut_ptr() as *mut pthread_mutexattr_t)};
+    res = unsafe{pthread_mutex_init(mutex, &lock_attr)};
     if res != 0 {
-        return Err(From::from(format!("pthread_condattr_init() : Failed with {}", res)));
+        return Err(SharedMemError::FailedToCreateLock(res as u32));
     }
     Ok(())
 }
 
-fn mutex_lock(mutex: *mut pthread_mutex_t, abs_timeout_time: &timespec) -> Result<()> {
+fn mutex_lock(mutex: *mut pthread_mutex_t, abs_timeout_time: &timespec) -> Result<(), SharedMemError> {
 
     let res: libc::c_int;
 
     if abs_timeout_time.tv_sec == -1 {
         res = unsafe {pthread_mutex_lock(mutex)};
         if res != 0 {
-            return Err(From::from("pthread_mutex_lock() : Error acquiring mutex"));
+            return Err(SharedMemError::FailedToAcquireLock(res as u32));
         }
         return Ok(())
     }
@@ -329,18 +331,18 @@ fn mutex_lock(mutex: *mut pthread_mutex_t, abs_timeout_time: &timespec) -> Resul
     if res == 0 {
         Ok(())
     } else if res == libc::ETIMEDOUT {
-        Err(From::from("pthread_mutex_timedlock() : Timed out"))
+        Err(SharedMemError::Timeout)
     } else {
-        Err(From::from(format!("pthread_mutex_timedlock() : Error {}", res)))
+        Err(SharedMemError::FailedToAcquireLock(res as u32))
     }
 }
 
-fn mutex_unlock(mutex: *mut pthread_mutex_t) -> Result<()> {
+fn mutex_unlock(mutex: *mut pthread_mutex_t) -> Result<(), SharedMemError> {
 
     let res: libc::c_int = unsafe {pthread_mutex_unlock(mutex)};
 
     if res != 0 {
-        Err(From::from(format!("mutex_unlock() : Error {}", res)))
+        Err(SharedMemError::FailedToAcquireLock(res as u32))
     } else {
         Ok(())
     }
@@ -353,7 +355,7 @@ impl LockImpl for Mutex {
     fn size_of(&self) -> usize {
         size_of::<pthread_mutex_t>()
     }
-    fn init(&self, lock_info: &mut GenericLock, create_new: bool) -> Result<()> {
+    fn init(&self, lock_info: &mut GenericLock, create_new: bool) -> Result<(), SharedMemError> {
         //Nothing to do if we're not the creator
         if !create_new {
             return Ok(());
@@ -362,16 +364,16 @@ impl LockImpl for Mutex {
         new_mutex(lock_info.lock_ptr as *mut pthread_mutex_t)
     }
     fn destroy(&self, _lock_info: &mut GenericLock) {}
-    fn rlock(&self, lock_ptr: *mut c_void) -> Result<()> {
+    fn rlock(&self, lock_ptr: *mut c_void) -> Result<(), SharedMemError> {
         mutex_lock(lock_ptr as *mut pthread_mutex_t, &timeout_to_abstimespec(Timeout::Infinite))
     }
-    fn wlock(&self, lock_ptr: *mut c_void) -> Result<()> {
+    fn wlock(&self, lock_ptr: *mut c_void) -> Result<(), SharedMemError> {
         mutex_lock(lock_ptr as *mut pthread_mutex_t, &timeout_to_abstimespec(Timeout::Infinite))
     }
-    fn runlock(&self, lock_ptr: *mut c_void) -> () {
+    fn runlock(&self, lock_ptr: *mut c_void) {
         match mutex_unlock(lock_ptr as *mut pthread_mutex_t) {_=>{},};
     }
-    fn wunlock(&self, lock_ptr: *mut c_void) -> () {
+    fn wunlock(&self, lock_ptr: *mut c_void) {
         match mutex_unlock(lock_ptr as *mut pthread_mutex_t) {_=>{},};
     }
 }
@@ -383,41 +385,41 @@ impl LockImpl for RwLock {
     fn size_of(&self) -> usize {
         size_of::<pthread_rwlock_t>()
     }
-    fn init(&self, lock_info: &mut GenericLock, create_new: bool) -> Result<()> {
+    fn init(&self, lock_info: &mut GenericLock, create_new: bool) -> Result<(), SharedMemError> {
         //Nothing to do if we're not the creator
         if !create_new {
             return Ok(());
         }
 
-        let mut lock_attr: [u8; size_of::<pthread_rwlockattr_t>()] = [0; size_of::<pthread_rwlockattr_t>()];
+        let mut lock_attr: pthread_rwlockattr_t = unsafe{std::mem::zeroed()};
         unsafe {
           //Set the PTHREAD_PROCESS_SHARED attribute on our rwlock
-          pthread_rwlockattr_init(lock_attr.as_mut_ptr() as *mut pthread_rwlockattr_t);
-          pthread_rwlockattr_setpshared(lock_attr.as_mut_ptr() as *mut pthread_rwlockattr_t, PTHREAD_PROCESS_SHARED);
+          pthread_rwlockattr_init(&mut lock_attr);
+          pthread_rwlockattr_setpshared(&mut lock_attr, PTHREAD_PROCESS_SHARED);
           //Init the rwlock
-          pthread_rwlock_init(lock_info.lock_ptr as *mut pthread_rwlock_t, lock_attr.as_mut_ptr() as *mut pthread_rwlockattr_t);
+          pthread_rwlock_init(lock_info.lock_ptr as *mut pthread_rwlock_t, &lock_attr);
         }
         Ok(())
     }
     fn destroy(&self, _lock_info: &mut GenericLock) {}
-    fn rlock(&self, lock_ptr: *mut c_void) -> Result<()> {
+    fn rlock(&self, lock_ptr: *mut c_void) -> Result<(), SharedMemError> {
         unsafe {
             pthread_rwlock_rdlock(lock_ptr as *mut pthread_rwlock_t);
         }
         Ok(())
     }
-    fn wlock(&self, lock_ptr: *mut c_void) -> Result<()> {
+    fn wlock(&self, lock_ptr: *mut c_void) -> Result<(), SharedMemError> {
         unsafe {
             pthread_rwlock_wrlock(lock_ptr as *mut pthread_rwlock_t);
         }
         Ok(())
     }
-    fn runlock(&self, lock_ptr: *mut c_void) -> () {
+    fn runlock(&self, lock_ptr: *mut c_void) {
         unsafe {
             pthread_rwlock_unlock(lock_ptr as *mut pthread_rwlock_t);
         }
     }
-    fn wunlock(&self, lock_ptr: *mut c_void) -> () {
+    fn wunlock(&self, lock_ptr: *mut c_void) {
         unsafe {
             pthread_rwlock_unlock(lock_ptr as *mut pthread_rwlock_t);
         }
@@ -426,34 +428,34 @@ impl LockImpl for RwLock {
 
 /* Event implementations */
 
-fn new_eventcond(event: &mut EventCond) -> Result<()> {
+fn new_eventcond(event: &mut EventCond) -> Result<(), SharedMemError> {
     /* Init signal state */
     event.signaled = false;
     let mut res: libc::c_int;
 
     /* Init the pthread_cond */
-    let mut cond_attr: [u8; size_of::<pthread_condattr_t>()] = [0; size_of::<pthread_condattr_t>()];
+    let mut cond_attr: pthread_condattr_t = unsafe {std::mem::zeroed()};
 
     //Set the PTHREAD_PROCESS_SHARED attribute for our pthread_cond
-    res = unsafe {pthread_condattr_init(cond_attr.as_mut_ptr() as *mut pthread_condattr_t)};
+    res = unsafe {pthread_condattr_init(&mut cond_attr)};
     if res != 0 {
-        return Err(From::from(format!("pthread_condattr_init() : Failed with {}", res)));
+        return Err(SharedMemError::FailedToCreateEvent(res as u32));
     }
-    res = unsafe {pthread_condattr_setpshared(cond_attr.as_mut_ptr() as *mut pthread_condattr_t, PTHREAD_PROCESS_SHARED)};
+    res = unsafe {pthread_condattr_setpshared(&mut cond_attr, PTHREAD_PROCESS_SHARED)};
     if res != 0 {
-        return Err(From::from(format!("pthread_condattr_setpshared() : Failed with {}", res)));
+        return Err(SharedMemError::FailedToCreateEvent(res as u32));
     }
     //Init the pthread_cond
-    res = unsafe {pthread_cond_init(&mut event.cond, cond_attr.as_mut_ptr() as *mut pthread_condattr_t)};
+    res = unsafe {pthread_cond_init(&mut event.cond, &cond_attr)};
     if res != 0 {
-        return Err(From::from(format!("pthread_cond_init() : Failed with {}", res)));
+        return Err(SharedMemError::FailedToCreateEvent(res as u32));
     }
 
     /* Init the pthread_mutex */
     new_mutex(&mut event.mutex)
 }
 
-fn event_wait(event: &mut EventCond, abs_timeout_time: &timespec, auto: bool) -> Result<()> {
+fn event_wait(event: &mut EventCond, abs_timeout_time: &timespec, auto: bool) -> Result<(), SharedMemError> {
     let mut res: libc::c_int = 0;
 
     //Lock mutex for our pthread_cond
@@ -480,15 +482,15 @@ fn event_wait(event: &mut EventCond, abs_timeout_time: &timespec, auto: bool) ->
     match mutex_unlock(&mut event.mutex) {_=>{},};
 
     if res == libc::ETIMEDOUT {
-        Err(From::from("pthread_cond_timedwait() : Time out"))
+        Err(SharedMemError::Timeout)
     } else if res != 0 {
-        Err(From::from(format!("pthread_cond_[timed]wait() : Error {}", res)))
+        Err(SharedMemError::FailedToSignalEvent(res as u32))
     } else {
         Ok(())
     }
 }
 
-fn event_set(event: &mut EventCond, state: EventState, abs_timeout_time: &timespec, auto: bool) -> Result<()> {
+fn event_set(event: &mut EventCond, state: EventState, abs_timeout_time: &timespec, auto: bool) -> Result<(), SharedMemError> {
 
     mutex_lock(&mut event.mutex, abs_timeout_time)?;
     match state {
@@ -507,7 +509,7 @@ fn event_set(event: &mut EventCond, state: EventState, abs_timeout_time: &timesp
 
             if res != 0 {
                 mutex_unlock(&mut event.mutex)?;
-                return Err(From::from(format!("pthread_cond_[signal|broadcast]() : Failed with err {}", res)));
+                return Err(SharedMemError::FailedToSignalEvent(res as u32));
             }
         }
     };
@@ -529,7 +531,7 @@ impl EventImpl for AutoGeneric {
         size_of::<EventCond>()
     }
     ///Initializes the event
-    fn init(&self, event_info: &mut GenericEvent, create_new: bool) -> Result<()> {
+    fn init(&self, event_info: &mut GenericEvent, create_new: bool) -> Result<(), SharedMemError> {
 
         //Nothing to do if we're not the creator
         if !create_new {
@@ -545,13 +547,13 @@ impl EventImpl for AutoGeneric {
         //Nothing to do here
     }
     ///This method should only return once the event is signaled
-    fn wait(&self, event_ptr: *mut c_void, timeout: Timeout) -> Result<()> {
+    fn wait(&self, event_ptr: *mut c_void, timeout: Timeout) -> Result<(), SharedMemError> {
         let event: &mut EventCond = unsafe {&mut (*(event_ptr as *mut EventCond))};
         //Wait for the event, automatically reset signal state
         event_wait(event, &timeout_to_abstimespec(timeout), true)
     }
     ///This method sets the event. This should never block
-    fn set(&self, event_ptr: *mut c_void, state: EventState) -> Result<()> {
+    fn set(&self, event_ptr: *mut c_void, state: EventState) -> Result<(), SharedMemError> {
         let event: &mut EventCond = unsafe {&mut (*(event_ptr as *mut EventCond))};
         //Set event using pthread_cond_signal
         event_set(event, state, &timeout_to_abstimespec(Timeout::Infinite), true)
@@ -566,7 +568,7 @@ impl EventImpl for ManualGeneric {
         size_of::<EventCond>()
     }
     ///Initializes the event
-    fn init(&self, event_info: &mut GenericEvent, create_new: bool) -> Result<()> {
+    fn init(&self, event_info: &mut GenericEvent, create_new: bool) -> Result<(), SharedMemError> {
 
         //Nothing to do if we're not the creator
         if !create_new {
@@ -582,13 +584,13 @@ impl EventImpl for ManualGeneric {
         //Nothing to do here
     }
     ///This method should only return once the event is signaled
-    fn wait(&self, event_ptr: *mut c_void, timeout: Timeout) -> Result<()> {
+    fn wait(&self, event_ptr: *mut c_void, timeout: Timeout) -> Result<(), SharedMemError> {
         let event: &mut EventCond = unsafe {&mut (*(event_ptr as *mut EventCond))};
         //Wait for the event, dont reset signal state
         event_wait(event, &timeout_to_abstimespec(timeout), false)
     }
     ///This method sets the event. This should never block
-    fn set(&self, event_ptr: *mut c_void, state: EventState) -> Result<()> {
+    fn set(&self, event_ptr: *mut c_void, state: EventState) -> Result<(), SharedMemError> {
         let event: &mut EventCond = unsafe {&mut (*(event_ptr as *mut EventCond))};
         //Set event using pthread_cond_broadcast
         event_set(event, state, &timeout_to_abstimespec(Timeout::Infinite), false)
